/*

	MPEGPIO.H
	
	Diamond Systems Corporation Universal Driver
	Version 7.00

	(c) Diamond Systems Corporation 2013
	
	http://www.diamondsystems.com

	(c) Copyright 2013 Diamond Systems Corporation. Use of this source code
	is subject to the terms of Diamond Systems' Software License Agreement.
	Diamond Systems provides no warranty of proper performance if this
	source code is modified.
*/


#ifndef _DSC_MPEGPIO_H
#define _DSC_MPEGPIO_H

#include "common.h"

#ifdef DSC_MPEGPIO_SUPPORT 
#ifdef __cplusplus
extern "C" {
#endif
///////////////////////
// MPEGPIO Constants //
///////////////////////

#define MPEGPIO_VENDORID				0x4453
#define MPEGPIO_DEVICEID				0x0B00

//////////////////////////////////////////////
// INTERRUPT BLOCK DEFINES
//////////////////////////////////////////////
#define MPEGPIO_INTERRUPT_BLOCK_OFFSET   0x70
#define MPEGPIO_INTERRUPT_ENABLE_OFFSET	 MPEGPIO_INTERRUPT_BLOCK_OFFSET + 0
#define MPEGPIO_INTERRUPT_INTCLR_OFFSET  MPEGPIO_INTERRUPT_BLOCK_OFFSET + 1
#define MPEGPIO_PIC_CONFIG_OFFSET		 (MPEGPIO_INTERRUPT_BLOCK_OFFSET+6)
#define MPEGPIO_FPGAIDMinor_OFFSET		(MPEGPIO_INTERRUPT_BLOCK_OFFSET+8)
#define MPEGPIO_FPGAIDMajor_OFFSET	  	(MPEGPIO_INTERRUPT_BLOCK_OFFSET+9)
#define MPEGPIO_FPGARevMinor_OFFSET	  	(MPEGPIO_INTERRUPT_BLOCK_OFFSET+10)
#define MPEGPIO_FPGARevMajor_OFFSET	  	(MPEGPIO_INTERRUPT_BLOCK_OFFSET+11)
#define MPEGPIO_BoardIDMinor_OFFSET	  	(MPEGPIO_INTERRUPT_BLOCK_OFFSET+12)
#define MPEGPIO_BoardIDMajor_OFFSET	  	(MPEGPIO_INTERRUPT_BLOCK_OFFSET+13)
#define MPEGPIO_BoardRevMinor_OFFSET  	(MPEGPIO_INTERRUPT_BLOCK_OFFSET+14)
#define MPEGPIO_BoardRevMajor_OFFSET  	(MPEGPIO_INTERRUPT_BLOCK_OFFSET+15)
#define MPEGPIO_FPGA_ID_MAJOR_VALUE		0x0B
#define MPEGPIO_INTERRUPT_BRDRST_OFFSET  MPEGPIO_INTERRUPT_BLOCK_OFFSET+15
#define MPEGPIO_INTERRUPT_BRDRST_BIT	7  
#define MPEGPIO_LED_OFFSET		 MPEGPIO_INTERRUPT_BLOCK_OFFSET+4

////////////////////////
// DIO BLOCK DEFINES
////////////////////////
#define MPEGPIO_DIO_BLOCK_OFFSET    0x20
#define MPEGPIO_MAX_DIO_PORTS       6
#define MPEGPIO_DIO_DIR_OFFSET		MPEGPIO_DIO_BLOCK_OFFSET+0x08
#define MPEGPIO_BITS_IN_BYTE_MAX    7
#define MPEGPIO_UNSIGNED_BYTE_MAX  255
#define MPEGPIO_BIT_HIGH	    1
#define MPEGPIO_BIT_LOW 	    0


//////////////////////////////////////////////
// COUNTER BLOCK DEFINES- COUNTER/PWM/WDT
//////////////////////////////////////////////
#define MPEGPIO_COUNTER_BLOCK_OFFSET				0x30
#define MPEGPIO_COUNTER_DATA_BYTE0					MPEGPIO_COUNTER_BLOCK_OFFSET+0
#define MPEGPIO_COUNTER_DATA_BYTE1					MPEGPIO_COUNTER_BLOCK_OFFSET+1
#define MPEGPIO_COUNTER_DATA_BYTE2					MPEGPIO_COUNTER_BLOCK_OFFSET+2
#define MPEGPIO_COUNTER_DATA_BYTE3					MPEGPIO_COUNTER_BLOCK_OFFSET+3
#define MPEGPIO_COUNTER_NUMBER_REGISTER				MPEGPIO_COUNTER_BLOCK_OFFSET+4
#define MPEGPIO_COUNTER_CMD_REGISTER				MPEGPIO_COUNTER_BLOCK_OFFSET+5
#define MPEGPIO_COUNTER_OUTPUT_ENABLE_DISABLE_OFFSET MPEGPIO_COUNTER_BLOCK_OFFSET+0x07

#define MPEGPIO_COUNTER_CMD_CLEAR					0x00
#define MPEGPIO_COUNTER_CMD_LOAD					0x10
#define MPEGPIO_COUNTER_CMD_SEL_DIRECTION_DOWN		0x20
#define MPEGPIO_COUNTER_CMD_SEL_DIRECTION_UP		0x21
#define MPEGPIO_COUNTER_CMD_EXTGATE_DISABLE			0x30
#define MPEGPIO_COUNTER_CMD_EXTGATE_ENABLE			0x31
#define MPEGPIO_COUNTER_CMD_COUNTER_DISABLE			0x40
#define MPEGPIO_COUNTER_CMD_COUNTER_ENABLE			0x41
#define MPEGPIO_COUNTER_CMD_COUNTER_OUTPUT_ENABLE		0x82
#define MPEGPIO_COUNTER_CMD_COUNTER_OUTPUT_DISABLE	0x80 //Both enable/disable commands upper bits are same
#define MPEGPIO_COUNTER_CMD_COUNTER_LATCH			0x50
#define MPEGPIO_COUNTER_CMD_RESET_ALL				0xF1
#define MPEGPIO_COUNTER_CMD_RESET_ONE				0xF0
#define MPEGPIO_COUNTER_CMD_CLKSELECT				0x60
#define MPEGPIO_COUNTER_CMD_CLKSELECT_EXT			0x60
#define MPEGPIO_COUNTER_CMD_CLKSELECT_50MHZ			0x62
#define MPEGPIO_COUNTER_CMD_CLKSELECT_1MHZ			0x63
#define MPEGPIO_COUNTER_CMD_OUTPULSE_WIDTH	    0x90
#define MPEGPIO_COUNTER_DISABLE_OUTPUT              0x80
#define MPEGPIO_COUNTER_AUTO_RELOAD_ENABLE          0x71
#define MPEGPIO_COUNTER_AUTO_RELOAD_SET             0x70
#define MPEGPIO_COUNTER_CLKSRC0_BIT					0
#define MPEGPIO_COUNTER_CLKFRQ0_BIT					1
#define MPEGPIO_COUNTER_COUNTER0					0
#define MPEGPIO_COUNTER_COUNTER1					1
#define MPEGPIO_COUNTER_50MHZ        				 50000000
#define MPEGPIO_COUNTER_1MHZ         				 1000000


// PWM command masks
#define MPEGPIO_PWM_BLOCK_OFFSET					0x30
#define MPEGPIO_PWM_CMD_REG	   						MPEGPIO_PWM_BLOCK_OFFSET+0xB
#define MPEGPIO_PWM_DATA_LSB  						MPEGPIO_PWM_BLOCK_OFFSET+0x08
#define MPEGPIO_PWM_DATA_CSB  						MPEGPIO_PWM_BLOCK_OFFSET+0x09
#define MPEGPIO_PWM_DATA_MSB  						MPEGPIO_PWM_BLOCK_OFFSET+0x0A
#define MPEGPIO_PWM_LOAD_C0_COUNTER					0x10
#define MPEGPIO_PWM_LOAD_C1_COUNTER					0x18
#define MPEGPIO_PWM_POLARITY_HIGH					0x20
#define MPEGPIO_PWM_POLARITY_LOW					0x28
#define MPEGPIO_PWM_DISABLE_PULSE_OUTPUT			0x30
#define MPEGPIO_PWM_ENABLE_PULSE_OUTPUT				0x38
#define MPEGPIO_PWM_STOP_CURRENT_PWM 				0x08
#define MPEGPIO_PWM_STOP_ALL_PWMS					0x00
#define MPEGPIO_PWM_DISABLE_OUTPUT_ON_DIO_PORT_F		0x50
#define MPEGPIO_PWM_ENABLE_OUTPUT_ON_DIO_PORT_F			0x58
#define MPEGPIO_PWM_SELECT_50MHZ_CLOCK_SOURCE			0x60
#define MPEGPIO_PWM_SELECT_1MHZ_CLOCK_SOURCE			0x68
#define MPEGPIO_PWM_START_CURRENT_PWM				0x70
#define MPEGPIO_PWM_START_ALL_PWMS					0x7F
#define MPEGPIO_PWM_SELECTED_MASK					0x07
#define MPEGPIO_PWM_COUNTING_PERIOD_MAX				16777215.0
#define MPEGPIO_COUNTER_50MHZ						50000000
#define MPEGPIO_COUNTER_1MHZ  						1000000
#define MPEGPIO_PWM_POLARITY_HIGH					0x20
#define MPEGPIO_PWM_POLARITY_LOW					0x28
#define MPEGPIO_PWM_ENABLE             				0x38
#define MPEGPIO_PWM_START              				0x70
#define MPEGPIO_MAX_PWM						   		 04
#define MPEGPIO_PWM_STOP_SELECTED_PWM  				0x08
#define MPEGPIO_PWM_STOP_ALL_PWM					0x00
#define MPEGPIO_PWM_RESET_ONE						0x40
#define MPEGPIO_PWM_RESET_ALL						0x48				




//MPEGPIOINIT

typedef struct
{
		int Address;        /*Base address assigned to board in PCI I/O memory space*/
		int IRQ;			/*IRQ number assigned to board*/
		int FPGAIDMajor	;	/*FPGA ID major value*/
		int FPGAIDMinor;	/*FPGA ID minor value*/
		int FPGARevMajor;	/*FPGA Revision major value*/
		int FPGARevMinor;	/*FPGA Revision minor value*/
		int BoardIDMajor;	/*Board ID major value*/
		int BoardIDMinor;	/*Board ID minor value*/
		int BoardRevMajor;	/*Board Revision major value*/
		int BoardRevMinor;	/*Board Revision minor value*/

}MPEGPIOINIT;

//MPEGPIOCTR

typedef struct
{
	int 			Num;			//Counter number, 0-7
	float 			Rate;		//Desired output frequency, Hz
	unsigned long 	Data;		//Initial load data, 32-bit unsigned binary
	int 			Clock;		//Clock source, must be 2 or 3 (see FPGA specification for usage)
	int 			OutEn;		//1 = enable output onto corresponding I/O pin; 0 = disable output
	int 			OutPol;		//1 = output pulses high, 0 = output pulses low; only used if CtrOutEn = 1
	int 			CountDir;	//0 = down counting, 1 = up  counting
	int 			Reload	;	//0 = one-shot counting, 1 = auto-reload (repetitive counting, only works in count down mode)
	int 			Cmd;			//Counter command, 0-15 (see FPGA specification for available commands)
	int 			CmdData	;	//Auxiliary data for counter command, 0-3 (see FPGA specification for usage)
	int 			ctrOutWidth ; // 0-3   0 = 1 clocks, 1 = 10 clocks, 2 = 100 clocks, 3 = 1000 clocks , only used if  CtrOutEn = 1 and CtrClock = 2 or 3

}MPEGPIOCTR;

//MPEGPIOPWM

typedef struct
{
	int Num;				/*PWM number, 0-3*/
	float Rate;				/*output frequency in Hz*/
	float Duty;				/*initial duty cycle, 0-100*/
	int Polarity;			/*0 = pulse high, 1 = pulse low*/
	int OutputEnable;		/*0 = disable output, 1 = enable output on DIO pin*/
	int Run	;				/*0 = don't start PWM, 1 = start PWM*/
	int Command;			/*0-15 = PWM command*/
	int CmdData;			/*0 or 1 for auxiliary PWM command data (used for certain commands)*/
	unsigned long Divisor;	/*24-bit value for use with period and duty cycle commands*/

}MPEGPIOPWM;

//MPEGPIOUSERINT

typedef struct
{
		void (*IntFunc) (void* parameter);		/*pointer to user function to run when interrupts occur*/
		int Mode;					/*0 = alone, 1 = before standard function, 2 = after standard function; the only valid mode is 0*/
		
		int Source;		//1-5; 1 = digital I/O; 2-5 = counter/timer 0-3 output
		int Enable;		//0 = disable interrupts, 1 = enable interrupts

}MPEGPIOUSERINT;

/////////////////////////
// Function Prototypes //
/////////////////////////
BYTE DSCUDAPICALL MPEGPIOMain(DSCB board, WORD func, void * params);

BYTE DSCUDAPICALL MPEGPIODIOConfig(BoardInfo* bi, int Port, int Config);
BYTE DSCUDAPICALL MPEGPIODIOConfigAll(BoardInfo* bi, int* Config);
BYTE DSCUDAPICALL MPEGPIODIOOutputByte(BoardInfo* bi, int Port, int Data);
BYTE DSCUDAPICALL MPEGPIODIOInputByte(BoardInfo* bi, int port, int* data);
BYTE DSCUDAPICALL MPEGPIODIOOutputBit(BoardInfo* bi, int Port, int Bit, int Value);
BYTE DSCUDAPICALL MPEGPIODIOInputBit(BoardInfo* bi, int port, int bit, int* value);

BYTE DSCUDAPICALL MPEGPIOCounterSetRate(BoardInfo* bi, MPEGPIOCTR *Ctr);
BYTE DSCUDAPICALL MPEGPIOCounterConfig(BoardInfo* bi, MPEGPIOCTR *Ctr);
BYTE DSCUDAPICALL MPEGPIOCounterRead(BoardInfo* bi, MPEGPIOCTR* Ctr);
BYTE DSCUDAPICALL MPEGPIOCounterReset(BoardInfo* bi, int CtrNum);
BYTE DSCUDAPICALL MPEGPIOCounterFunction(BoardInfo* bi, MPEGPIOCTR* Ctr);

BYTE DSCUDAPICALL MPEGPIOPWMConfig(BoardInfo* bi, MPEGPIOPWM* MPEGPIOpwm);
BYTE DSCUDAPICALL MPEGPIOPWMStart(BoardInfo* bi, int Num);
BYTE DSCUDAPICALL MPEGPIOPWMStop(BoardInfo* bi, int Num);
BYTE DSCUDAPICALL MPEGPIOPWMReset(BoardInfo* bi, int Num);
BYTE DSCUDAPICALL MPEGPIOPWMCommand(BoardInfo* bi, MPEGPIOPWM* MPEGPIOpwm);

BYTE DSCUDAPICALL MPEGPIOUserInterruptConfig(BoardInfo* bi, MPEGPIOUSERINT* Userint);
BYTE DSCUDAPICALL MPEGPIOUserInterruptRun(BoardInfo* bi, int Source);
BYTE DSCUDAPICALL MPEGPIOUserInterruptCancel(BoardInfo* bi, int Source);

BYTE DSCUDAPICALL MPEGPIOInitBoard(DSCCBP* dsccbp, MPEGPIOINIT *Init);
BYTE DSCUDAPICALL MPEGPIOFreeBoard(DSCB board);
BYTE DSCUDAPICALL MPEGPIOLED(BoardInfo* bi, int State);
BYTE DSCUDAPICALL MPEGPIOConfig(BoardInfo* bi, int Value);
BYTE DSCUDAPICALL DSCRegisterRead(BoardInfo* bi, int Page, int Register, int* Value);
BYTE DSCUDAPICALL DSCRegisterWrite(BoardInfo* bi, int Page, int Register, int Value);
BYTE MPEGPIOInputLong ( BoardInfo* bi, DWORD start_address, DWORD* data );
BYTE MPEGPIOOutputLong ( BoardInfo* bi, DWORD start_address, DWORD data );



#else //tells people that there's no MPEGPIO


#endif

#ifdef __cplusplus
} /* Closes the extern "C" */
#endif

#endif //#ifndef _DSC_MPEGPIO_H


