//=============================================================================
// (c) Copyright 2014 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: MPEGPIOCounterFunction.c   v1.00
//Dependency :	Universal Driver version 7.00
//				DSCUD.H - Include file.
//				
//				LIBDSCUD-7.00.a	- Library file for Linux 2.6.29-3.x
//						
//				DSCUD.LIB	- Static library file for Windows 
//						( USE VC++ 6.0 or Visual Studio 2005 )
//				
// Description: This demo application demonstrates the usage of counter in up and down mode configuration.  
//		
//============================================================================
#include "DSCUD_demo_def.h"
#include "mpegpio.h"

//Global variable declarations
ERRPARAMS errorParams; //structure for returning error code and error string
DSCCBP dsccbp; //structure containing PCI board settings
BoardInfo *bi=NULL; //Structure containing board base address
MPEGPIOINIT Init; //structure containing board FPGA revision ID, major ID etc.
MPEGPIOCTR counter; //Structure containing counter settings
//=============================================================================
// Name: main()
// Desc: It calls universal driver counter related functions to demonstrates use of counter
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. Program Counter
//	    IV. Clean up
//
//=============================================================================

int main ( void )
{
	int intBuff = 0; // temp variables of size int
	int temp = 0; // temporary storage
   	char input_buffer[20];
   	float frequency = 0.0;
	float clkfrequency=0;

   	memset ( &dsccbp, 0, sizeof(DSCCBP) );
		  
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//    STEPS TO FOLLOW:
	//
	//	  1. initialize the driver, using the driver version for validation
	//=========================================================================

	if ( (dscInit ( DSC_VERSION ) != DE_NONE) )
	{
		dscGetLastError ( &errorParams );
		printf ( "dscInit error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
		return 0;
	}
	
	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the DS-MPE-GPIO Board. This function passes the various
	//	   hardware parameters to the driver and initializes the hardware.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. set the board type to DSC_MPEGPIO for the DS-MPE-GPIO board
	//	   2. Set the pci_slot variable to zero 
	//     3. Call MPEGPIOInitBoard universal driver function to initialize the board
	//=========================================================================

	printf ( "\nDSC MPE-GPIO COUNTER FUNCTIONS DEMO\n" );

	dsccbp.boardtype = DSC_MPEGPIO;
	dsccbp.pci_slot  = 0;
	 
	if(MPEGPIOInitBoard ( &dsccbp ,&Init ) !=DE_NONE)
	{
		dscGetLastError(&errorParams);
		printf("MPEGPIOInitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
		return 0;
	}
	bi = DSCGetBoardInfo ( dsccbp.boardnum ); 
	//=========================================================================
	// III. PROGRAM COUNTER
	//
	//	    MPEGPIOCounterConfig() function configures the Counter circuit 
	// 		in count up/down direction based on user input,then reads the counter value and displays every second.
	//
	//=========================================================================
	do
	{
		printf("Enter counter number (0-7; default:0):");
		fgets(input_buffer,20,stdin);
		if(input_buffer[0]=='\n')
		{	
			counter.Num = 0;
		}
		else
		{
			sscanf(input_buffer,"%d",&counter.Num);
		}

		printf("Enter Counter Direction (0 = down counting, 1 = up  counting; default = 0):");
		fgets(input_buffer,20,stdin);
		if(input_buffer[0]=='\n')
		{	
			counter.CountDir = 0;
		}
		else
		{
			sscanf(input_buffer,"%d",&counter.CountDir);
		}
		
		if(counter.CountDir == 0)
		{
			printf("Select Clock source (0=External ,2=Internal clock 50MHz,3=Internal clock 1MHz; Default = 2):");
			fgets(input_buffer,20,stdin);
			if(input_buffer[0]=='\n')
			{	
				counter.Clock = 2;
			}
			else
			{
				sscanf(input_buffer,"%d",&counter.Clock);
			}

			printf("Enter Output Enable (0=disable, 1=Enable; default = 1):");
			fgets(input_buffer,20,stdin);
			if(input_buffer[0]=='\n')
			{	
				counter.OutEn = 1;
			}
			else
			{
				sscanf(input_buffer,"%d",&counter.OutEn);
			}
			
			if(counter.OutEn)
			{
				printf("Enter Output Polarity (0=negative, 1= positive; default = 1):");
				fgets(input_buffer,20,stdin);
				if(input_buffer[0]=='\n')
				{	
					counter.OutPol = 1;
				}
				else
				{
					sscanf(input_buffer,"%d",&counter.OutPol);
				}

				printf("Enter output pulse width (0-3 ,0 = 1 clocks, 1 = 10 clocks, 2 = 100 clocks, 3 = 1000 clocks, default = 3) : ");
				fgets(input_buffer,20,stdin);
				if(input_buffer[0]=='\n')
				{	
					counter.ctrOutWidth = 3;
				}
				else
				{
					sscanf(input_buffer,"%d",&counter.ctrOutWidth);
				}
			}

			printf("Enter Counter Frequency (default = 100)");
			fgets(input_buffer,20,stdin);
			if(input_buffer[0]=='\n')
			{	
				frequency = 100;
			}
			else
			{
				sscanf(input_buffer,"%f",&frequency);
			}
		}
		else
		{
			printf("Select Clock source (0=External ,2=Internal clock 50MHz,3=Internal clock 1MHz; Default = 0):");
			fgets(input_buffer,20,stdin);
			if(input_buffer[0]=='\n')
			{	
				counter.Clock = 0;
			}
			else
			{
				sscanf(input_buffer,"%d",&counter.Clock);
			}
			counter.OutEn = 0;
			counter.OutPol = 0;
		}
		
		counter.Reload = 1;

		if(counter.Clock == 2)
			counter.Data = (unsigned long)(50000000/frequency);
		else if(counter.Clock == 3)
			counter.Data = (unsigned long)(1000000/frequency);
		else if( (counter.Clock == 0) && (counter.CountDir == 1) )
		counter.Data = 0;
		else if((counter.Clock == 0) && (counter.CountDir == 0))
		{
		  printf("Enter external clock frequency : ");
		  fgets(input_buffer,20,stdin);
		  sscanf(input_buffer,"%f",&clkfrequency);
		  counter.Data =(unsigned long) (clkfrequency/frequency);	  
		}
		if( (counter.CountDir == 1) && (counter.Clock == 0))
		{
			if(MPEGPIODIOConfig(bi,3,0) !=DE_NONE)  //This is user responsibility to set the corresponding DIO pin to input mode. 
            {
                dscGetLastError(&errorParams);
				printf("MPEGPIODIOConfig error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
				return 0;
           	}
		}
		if( (counter.CountDir ==0)&& (counter.OutEn == 1) && ( (counter.Num >=0) && (counter.Num<=3) ) )
		{
			if(MPEGPIODIOConfig(bi,4,1) !=DE_NONE) //This is user responsibility to set the corresponding DIO pin to output mode. 
           	{
               	dscGetLastError(&errorParams);
				printf("MPEGPIODIOConfig error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
				return 0;
            }
		}
		if(MPEGPIOCounterConfig(bi,&counter) !=DE_NONE)
		{
			dscGetLastError(&errorParams);
			printf("MPEGPIOCounterConfig error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
			return 0;
		}
		printf("Press ENTER key to stop counting \n");
		
		while( !kbhit())
		{
			dscSleep(1000);
			MPEGPIOCounterRead(bi,&counter);
			printf("Counter Data %ld \r",counter.Data);
			fflush(stdout);
		}

		fgets(input_buffer,20,stdin);
		
		if(MPEGPIOCounterReset(bi,counter.Num) !=DE_NONE)
		{
			dscGetLastError(&errorParams);
			printf("MPEGPIOCounterReset error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
			return 0;
		}
		printf ("press Enter to repeat or 'q' to quit:");
		fgets ( input_buffer, 20, stdin );

    } while ( input_buffer[0] != 'q' );
	//=========================================================================
	// IV. CLEANUP
	//
	//	   Clean up any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the driver resources
	//=========================================================================

	dscFree ( );
	return 0;

} // end main()

