//=============================================================================
// (c) Copyright 2014 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: MPEGPIOCounterSetRate.c   v1.00
//Dependency :	Universal Driver version 7.00
//				DSCUD.H - Include file.
//				
//				LIBDSCUD-7.00.a	- Library file for Linux 2.6.29-3.x
//						
//				DSCUD.LIB	- Static library file for Windows 
//						( USE VC++ 6.0 or Visual Studio 2005 )
//				
// Description: This demo application demonstrates the usage of UD function  
//				MPEGPIOCounterSetRate(). It configures the counter in down mode and enabling the 
//				counter output on corresponding DIO pins is optional.
//============================================================================

#include "DSCUD_demo_def.h"
#include "mpegpio.h"

//Global variable declarations
ERRPARAMS errorParams; //structure for returning error code and error string
DSCCBP dsccbp; //structure containing PCI board settings
BoardInfo *bi=NULL; //Structure containing board base address
MPEGPIOINIT Init; //structure containing board FPGA revision ID, major ID etc.
MPEGPIOCTR counter;//Structure containing counter settings
//=============================================================================
// Name: main()
// Desc: It calls universal driver counter related functions to demonstrates use of counter
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. Program Counter
//	    IV. Clean up
//
//=============================================================================

int main ( void )
{

	int intBuff = 0; // temp variables of size int
	int temp = 0; // temporary storage
    char input_buffer[20];
    

   	memset ( &dsccbp, 0, sizeof(DSCCBP) );
		  
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//    STEPS TO FOLLOW:
	//
	//	  1. initialize the driver, using the driver version for validation
	//=========================================================================

	if ( (dscInit ( DSC_VERSION ) != DE_NONE) )
	{
		dscGetLastError ( &errorParams );
		printf ( "dscInit error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
		return 0;
	}
		
	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the DS-MPE-GPIO Board. This function passes the various
	//	   hardware parameters to the driver and initializes the hardware.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. set the board type to DSC_MPEGPIO for the DS-MPE-GPIO board
	//	   2. Set the pci_slot variable to zero 
	//     3. Call MPEGPIOInitBoard universal driver function to initialize the board
	//=========================================================================

	printf ( "\nDSC MPE-GPIO COUNTER FUNCTIONS DEMO\n" );

	dsccbp.boardtype = DSC_MPEGPIO;
	dsccbp.pci_slot  = 0;
	 
	if(MPEGPIOInitBoard ( &dsccbp ,&Init ) !=DE_NONE)
	{
		dscGetLastError(&errorParams);
		printf("MPEGPIOInitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
		return 0;
	}
	bi = DSCGetBoardInfo ( dsccbp.boardnum ); 
	
	//=========================================================================
	// III. PROGRAM COUNTER
	//
	//	    MPEGPIOCounterSetRate() function configures the Counter circuit in count down direction and
	//	    the counter output is enabled corresponding DIO based on user input, then reads the counter value and displays every second.
	//
	//=========================================================================
	do
	{
		printf("Enter  counter number (0-7)<default:0>:");
		fgets(input_buffer,20,stdin);
		if(input_buffer[0]=='\n')
		{	
			counter.Num = 0;
		}
		else
		{
			sscanf(input_buffer,"%d",&counter.Num);
		}
		
		printf("Enter Counter frequency rate (1-50MHz ) (Default = 1000)");
		fgets(input_buffer,20,stdin);
		if(input_buffer[0]=='\n')
		{	
			counter.Rate = 1000;
		}
		else
		{
			sscanf(input_buffer,"%f",&counter.Rate);
		}
		
		printf("Enter Output Enable (0=disable, 1=Enable; default = 1):");
		fgets(input_buffer,20,stdin);
		if(input_buffer[0]=='\n')
		{	
			counter.OutEn = 1;
		}
		else
		{
			sscanf(input_buffer,"%d",&counter.OutEn);
		}
		
		if(counter.OutEn)
		{
			printf("Enter Output Polarity (0=negative, 1= positive; default = 1):");
			fgets(input_buffer,20,stdin);
			if(input_buffer[0]=='\n')
			{	
				counter.OutPol = 1;
			}
			else
			{
				sscanf(input_buffer,"%d",&counter.OutPol);
			}

			printf("Enter output pulse width (0-3 ,0 = 1 clocks, 1 = 10 clocks, 2 = 100 clocks, 3 = 1000 clocks, default = 3) : ");
			fgets(input_buffer,20,stdin);
			if(input_buffer[0]=='\n')
			{	
				counter.ctrOutWidth = 3;
			}
			else
			{
				sscanf(input_buffer,"%d",&counter.ctrOutWidth);
			}
		}
	
		counter.CountDir ==0 ; // Down direction
		if( (counter.CountDir ==0)&& (counter.OutEn == 1) && ( (counter.Num >=0) && (counter.Num<=3) ) )
		{
			if(MPEGPIODIOConfig(bi,4,1) !=DE_NONE) //This is user responsibility to set the corresponding DIO pin to output mode 
           	{
					dscGetLastError(&errorParams);
					printf("MPEGPIODIOConfig error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
					return 0;
			}
        }
		
		if(MPEGPIOCounterSetRate(bi,&counter) !=DE_NONE)
		{
			dscGetLastError(&errorParams);
			printf("MPEGPIOCounterSetRate error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
			return 0;
		}
		printf("Press ENTER key to stop counting \n");
		
		while( !kbhit())
		{
			dscSleep(1000);
			MPEGPIOCounterRead(bi,&counter);
			printf("Counter Data %ld \r",counter.Data);
			fflush(stdout);
			
		}
		fgets(input_buffer,20,stdin);
				
		if(MPEGPIOCounterReset(bi,counter.Num) !=DE_NONE)
		{
			dscGetLastError(&errorParams);
			printf("MPEGPIOCounterReset error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
			return 0;
		}
		printf ("press Enter to repeat or 'q' to quit:");
		fgets ( input_buffer, 20, stdin );

    } while ( input_buffer[0] != 'q' );
	//=========================================================================
	// IV. CLEAN UP
	//
	//	   Clean up any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the driver resources
	//=========================================================================

	dscFree ( );
	
	return 0;

} // end main()

