//=============================================================================
// (c) Copyright 2014 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: MPEGPIODIOFunction.c  v1.00
//
// Dependency :	Universal Driver version 7.00
//				DSCUD.H - Include file.
//				
//				LIBDSCUD-7.00.a	- Library file for Linux 2.6.29-3.x
//						
//				DSCUD.LIB	- Static library file for Windows 
//						( USE VC++ 6.0 or Visual Studio 2005 )
//				 
// Description: This demo application shows how to configure DIO ports in input and output mode 
//				then read and write to the DIO ports.
//============================================================================
#include "DSCUD_demo_def.h"
#include "mpegpio.h" 

//DIO Port No
#define DIOPORT_A 0
#define DIOPORT_B 1
#define DIOPORT_C 2
#define DIOPORT_D 3
#define DIOPORT_E 4
#define DIOPORT_F 5

#define DIOPORT_MAX 6

//Global variable declarations
ERRPARAMS errorParams; // structure for returning error code and error string
DSCCBP dsccbp; //structure containing PCI board settings
int portConfig[6]; //used to store the port directions

//Function declarations
void configureDIOPort();
void configureAllDIOPort();
void writeByte2DIOPort();
void readByteDIOPort();
void writeBit2DIOPort();
void readBitDIOPort();
void DIOPortLoopbackTest();
void registerReadBack(int);


MPEGPIOINIT init; //structure containing board FPGA revision ID, major ID etc.
BoardInfo *bi; //Structure containing board base address
char input_buffer[20];


//=============================================================================
// Name: main()
// Desc: It calls universal driver DIO related functions to demonstrates use of DIO ports
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. DIO INPUT AND OUTPUT
//	    IV. Clean up
//
//=============================================================================

int main ( void )
{

	int intBuff = 0; // temp variables of size int
	int temp = 0; // temporary storage
    
   	memset ( &dsccbp, 0, sizeof(DSCCBP) );
	memset (portConfig,0, sizeof(int)*DIOPORT_MAX);

  
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//    STEPS TO FOLLOW:
	//
	//	  1. initialize the driver, using the driver version for validation
	//=========================================================================

	if ( (dscInit ( DSC_VERSION ) != DE_NONE) )
	{
		dscGetLastError ( &errorParams );
		printf ( "dscInit error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the DS-MPE-GPIO Board. This function passes the various
	//	   hardware parameters to the driver and initializes the hardware.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. set the board type to DSC_MPEGPIO for the DS-MPE-GPIO board
	//	   2. Set the pci_slot variable to zero 
	//     3. Call MPEGPIOInitBoard universal driver function to initialize the board
	//=========================================================================

	printf ( "\nDSC MPE-GPIO DIO FUNCTIONS DEMO\n" );

	dsccbp.boardtype = DSC_MPEGPIO;
    dsccbp.pci_slot  =0;

	if ( (MPEGPIOInitBoard (&dsccbp, &init )!= DE_NONE) )
	{
		dscGetLastError ( &errorParams );
		printf (  "MPEGPIOInitBoard error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
		return 0;
	}
	bi=DSCGetBoardInfo(dsccbp.boardnum);
	
 	//=========================================================================
	// III. DIO INPUT AND OUTPUT
	//      
	//      This section does various operations on DIO ports, the desired option
	//      can be selected from menu. 	
	//
	//		1) Configure a port
	//		2) Configure all DIO port
	//		3) Write a byte to a port
	//		4) Read a byte from a port
	//		5) Write a bit to a port 
	//		6) Read a bit from a port 
	//		7) Port loop back with 0-255 read/write repetitive
	//		8) Register page read back 
	//		q) Quit Program 
	// 		Choose these options in any order from the menu!
	//=========================================================================

	printf( "\nDIO INPUT AND OUTPUT: Main Menu \n" );

	do
	{
		intBuff = 0;
		printf (  "########################################################\n" );
		printf (  "Please Select an option from below!\n" );
		printf (  "1) Configure a port\n" );
		printf (  "2) Configure all DIO port\n");
		printf (  "3) Write a byte to a port\n" );
		printf (  "4) Read a byte from a port\n" );
		printf (  "5) Write a bit to a port \n" );
		printf (  "6) Read a bit from a port \n" );
		printf (  "7) Port loop back with 0-255 read/write repetitive\n" );
		printf (  "8) Register page read back \n" ); 
		printf (  "q) Quit Program \n");
		printf (  "########################################################\n\n" );


        fgets ( input_buffer, 20, stdin );
        sscanf ( input_buffer, "%d", &intBuff );

        if ( input_buffer[0] == 'q' )
        {
            intBuff = 'q';
        }

      
		switch ( intBuff )
		{
			case 1:
				configureDIOPort();
				break;
			case 2:
			    configureAllDIOPort();
				break;

			case 3:
				writeByte2DIOPort();
				break;

			case 4:
				readByteDIOPort();
				break;
			case 5:
				writeBit2DIOPort();
				break;

			case 6:
				readBitDIOPort();
				break;
				
			case 7:
				DIOPortLoopbackTest();
				break;
			case 8:
			   	registerReadBack(dsccbp.io_address[0]);
				break;
				
           	case 'q':
				return 0;
            	break;

			default: // invalid option selected
				printf("Please Enter valid option \r\n");
				break;

		};
 
		

	} while ( input_buffer[0] != 'q' );

	//=========================================================================
	// IV. CLEANUP
	//
	//	   Clean up any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the driver resources
	//=========================================================================

	dscFree ( );

	return 0;

} // end main()

//=============================================================================
// Name: configureDIOPort()
// Desc: Configure DIO ports A-F
// Parameters: void
// Return: void
//=============================================================================

void configureDIOPort()
{

	int intBuff = 0; // temp variables of size int
	BYTE port = 0; // digital I/O port
	BYTE config = 0; // DIO configuration byte to send

	printf ( "Info : Port number mapping (0- A, 1 - B, 2 - C, 3 - D, 4- E, 5 - F)\n");
    do
	{
		
		printf ( "\nEnter a port number to configure or Enter -1 to go to Main menu \n");
		fgets(input_buffer,20,stdin);
                sscanf(input_buffer,"%d",&intBuff);
		port = (BYTE) intBuff;

		if( intBuff==-1) break;
		
		if ( ((intBuff < DIOPORT_A) || (intBuff > DIOPORT_MAX)) )
		{
			printf ( "ERROR: Valid Port Numbers are: (0-5 = A-F)" );
			return;
		}

	
		switch ( port )
		{

			// byte-addressable
			case DIOPORT_A:
			case DIOPORT_B:
			case DIOPORT_D:
	                case DIOPORT_E:

				printf ( "Please enter a port direction (0 - Input, 1 - Output): " );
				fgets(input_buffer,20,stdin);
               	sscanf(input_buffer,"%d",&intBuff);

				config = (BYTE) intBuff;
                		portConfig[port]=intBuff;
				break;

			// bit-addressable
			case DIOPORT_C:
			case DIOPORT_F:

				printf ( "Please enter a port direction (0x0 - 0xF) in Hex (0 - Input, 1 - Output): ");
				fgets(input_buffer,20,stdin);
               			sscanf(input_buffer,"%x",&intBuff);
				config = (BYTE) intBuff;
				portConfig[port]=intBuff;
				break;

			default: // impossible
				break;

		};

		// Sets a port (A-F direction and mode settings for ports A-F
		if ( ( MPEGPIODIOConfig ( bi, port, config ) != DE_NONE) )
		{
			dscGetLastError ( &errorParams );
			printf("MPEGPIODIOConfig error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
			return;
		}
	intBuff=-1;	
    }while(1);
	return;

}
//=============================================================================
// Name: configureAllDIOPort()
// Desc: Configure DIO ports A-F
// Parameters: void
// Return: void
//=============================================================================
void configureAllDIOPort()
{
  int *config;
  int index=0;
  int intBuff = 0; // temp variables of size int
  config = (int *) malloc(sizeof(int)*DIOPORT_MAX);
  for(index =0 ;index <DIOPORT_MAX;index++)
  {
		if(index != 2 && index !=5)
		{
			printf ( "Please enter a port direction for PORT %d (0 - Input, 1 - Output): ", index);
			fgets(input_buffer,20,stdin);
			sscanf(input_buffer,"%d",&intBuff);
			portConfig[index]=intBuff;
			config[index]=intBuff;
		}
		else
		{
			printf ( "Please enter a port direction for PORT %d (0x0 - 0xF) in Hex (0 - Input, 1 - Output): ",index);
			fgets(input_buffer,20,stdin);
			sscanf(input_buffer,"%x",&intBuff);
			portConfig[index]=intBuff;
			config[index]=intBuff;
		}
}
  
  if(MPEGPIODIOConfigAll(bi,config) !=DE_NONE)
  {
			dscGetLastError ( &errorParams );
			printf("MPEGPIODIOConfigAll error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
			return;
  }
}
//=============================================================================
// Name: writeByte2DIOPort()
// Desc: Program a port A-F by sending a byte to the selected port's
//	  register.
// Parameters: void
// Return: void
//=============================================================================

void writeByte2DIOPort()
{

	int intBuff = 0;
	BYTE output_byte = 0;
	BYTE port = 0; 
	
    do
	{
		printf ( "\nEnter port number for output (0-%d): or Enter -1 to go to Main menu \n",DIOPORT_MAX-1);
		
		fgets(input_buffer,20,stdin);
        sscanf(input_buffer,"%d",&intBuff);
		port = (BYTE) intBuff;
		if( intBuff==-1) break;	
				
		if(portConfig[port]==0)
		{
		  printf("The port %d is not configured as output and the Port is in input mode \n",intBuff);
		  return;
		}
		if ( intBuff > DIOPORT_MAX || intBuff < 0 )
		{
			printf ( "ERROR: Valid Port Numbers are: (0-%d) ", DIOPORT_MAX );
			return;
		}

		printf ( "Enter the HEXADECIMAL value to port %d (example: 0xA5): ", port );
		fgets(input_buffer,20,stdin);
               	sscanf(input_buffer,"%x",&intBuff);
		output_byte = (BYTE) intBuff;

		if( (MPEGPIODIOOutputByte ( bi, port, output_byte ) != DE_NONE) )
		{
			dscGetLastError ( &errorParams );
			printf (  "MPEGPIODIOOutputByte() error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
			return;
		}

		
		
    } while(1);
	return;

}

//=============================================================================
// Name: readByteDIOPort()
// Desc: Read a port A-F by receiving a byte from the selected port's  register.
// Parameters:void
// Return: void
//=============================================================================

void readByteDIOPort()
{

	int intBuff = 0;
	BYTE port = 0; 
	int input_byte = 0; 
	int delay = 300;
	int repeat = 1;
	
	do
	{	
		printf ( "\nEnter port number for input (0-%d): or Enter -1 to go to Main menu \n", DIOPORT_MAX-1);
		fgets(input_buffer,20,stdin);
               	sscanf(input_buffer,"%d",&intBuff);
		port = (BYTE) intBuff;
      	if(intBuff == -1) 	break;

       
	 if((intBuff==DIOPORT_C || intBuff==DIOPORT_F)  )
        {
            if(portConfig[port]== 0xF)
	    {
                printf("The port %d is not configured as input and the Port is in output mode \n",intBuff);
		return ;
	    }
 	}
	else
	{
		
	  if( portConfig[port]==1)
	  {
		printf("The port %d is not configured as input and the Port is in output mode \n",intBuff);
		return;
	  }
	}
	if ( intBuff > DIOPORT_MAX || intBuff < 0 )
	{
		printf ( "ERROR: Valid Port Numbers are: (0-%d) ", DIOPORT_MAX );
		return;
	}

	printf("\nPress ENTER key to stop reading ...\r\n");
	

	do
	{

		if( (MPEGPIODIOInputByte ( bi, port, &input_byte ) != DE_NONE) )
		{
			dscGetLastError ( &errorParams );
			printf (  "MPEGPIODIOInputByte() error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
			return;
		}

		dscSleep( 1000 );

		printf("Port 0x%x : Byte value received from port = 0x%x\r\n", port, input_byte);


	}
	
	while ( !( kbhit() ) );
		
    }while(1);

	return;

}


//=============================================================================
// Name: writeBit2DIOPort()
// Desc: Program a port A-F by sending a digital value to the selected
// 	  port's bit register.
// Parameters:void
// Return: void
//=============================================================================
void writeBit2DIOPort()
{

	int intBuff = 0; 
	BYTE digital_value = 0;
         BYTE output_byte = 0; 
	BYTE port = 0; 
	BYTE bit = 0; 
	do
	{
		printf ( "\nEnter port number for output (0-%d): or Enter -1 to go to Main menu \n", DIOPORT_MAX-1);
		
		fgets(input_buffer,20,stdin);
               	sscanf(input_buffer,"%d",&intBuff);
		port = (BYTE) intBuff;
	        if(intBuff == -1) break;
		
		if(port == DIOPORT_C || port == DIOPORT_F)
		{
		
		   printf ( "Enter which bit to write (0-3): " );
		   fgets(input_buffer,20,stdin);
		   sscanf(input_buffer,"%d",&intBuff);
		   bit = (BYTE) intBuff;
		   if ( ((intBuff < 0) || (intBuff > 3)) )
		   {
			printf ( "ERROR: Valid bits are: (0-3) " );
			return;
		   }
		    if(((portConfig[port]) & (1<<bit)) == 0)
		    {
			      printf("The port %d and %d - bit is not configured as output and the Port %d - bit is in input mode \n",port,intBuff,bit);
			      return;
		    }
		}
		else 
		{
            if(port == DIOPORT_B || port == DIOPORT_E)
			{
			   printf ( "Enter which bit to write (0-5): " );
			   fgets(input_buffer,20,stdin);
		 	  sscanf(input_buffer,"%d",&intBuff);
		 	  bit = (BYTE) intBuff;
		  	 if ( ((intBuff < 0) || (intBuff > 5)) )
		  	 {
				printf ( "ERROR: Valid bits are: (0-5) " );
				return;
		  	 }
			}
			else
		        {
			   printf ( "Enter which bit to write (0-7): " );
			   fgets(input_buffer,20,stdin);
		 	    sscanf(input_buffer,"%d",&intBuff);
		 	   bit = (BYTE) intBuff;
		  	   if ( ((intBuff < 0) || (intBuff > 7)) )
		  	   {
				printf ( "ERROR: Valid bits are: (0-7) " );
				return;
		  	   }
			}
                       
			if(portConfig[port] == 0)
			{
			   printf("The port %d and %d-bit is not configured as output and the Port %d-bit is in input mode \n",port,intBuff,bit);
			 return;
			}
		}

		printf ("Enter bit value (0 or 1) " );
		fgets(input_buffer,20,stdin);
		sscanf(input_buffer,"%d",&intBuff);
		digital_value = (BYTE) intBuff;
		if ( ((intBuff < 0) || (intBuff > 1)) )
		{
			printf ( "ERROR: Valid bit values are: (0 and 1) " );
			return;
		}
		if ( (MPEGPIODIOOutputBit ( bi, port, bit, digital_value ) != DE_NONE) )
		{
			dscGetLastError ( &errorParams );
			printf("MPEGPIODIOOutputBit () error: %s %s\n",
				  dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
			return;
		}

	 } while(1);

	return;

}

//=============================================================================
// Name: readBitDIOPort()
// Desc: Read a port A-F by receving a digital value from the selected
// 	  port's bit register.
// Parameters: void
// Return: void
//=============================================================================

void readBitDIOPort()
{

	int intBuff = 0; // temp variables of size int
	int digital_value = 0; // digital representation of the bit
	BYTE port = 0; // digital I/O port
	BYTE bit = 0; // which bit to read

	do
	{
		printf ( "\nEnter port number for input (0-%d): or Enter -1 to go to Main menu \n", DIOPORT_MAX-1);
		fgets(input_buffer,20,stdin);
               	sscanf(input_buffer,"%d",&intBuff);
		port = (BYTE) intBuff;
        	if(intBuff== -1) break;
		if( ((port == DIOPORT_C) || (port == DIOPORT_F)) )
		{
			if((portConfig[port]==0xF))
			{		  
				printf("The port %d is not configured as input and the Port is in output mode\n",intBuff);
				return;
			}
		}
                else
		{
			if( portConfig[port]==1)
			{

				printf("The port %d is not configured as input and the Port is in output mode	\n",port);
				return;
			}
                 
		}

    printf("The port %d each bit values are \n",intBuff);
	printf("-------------------------------------------------------------------------------------------------------\r\n");
	printf("PortNo:  7   6   5   4   3   2   1   0   \r\n");
	printf("-------------------------------------------------------------------------------------------------------\r\n");
	printf("  %d   ", intBuff);
		
		for(intBuff=7 ;intBuff>=0;intBuff--)
		{
			bit = (BYTE) intBuff;

			if ( (MPEGPIODIOInputBit ( bi, port, bit, &digital_value ) != DE_NONE) )
			{
				dscGetLastError ( &errorParams );
				printf("MPEGPIODIOInputBit () error: %s %s\n",
					  dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
				return;
			}

		printf("   %x",digital_value);
		
		}

		printf("\n");

		
    }while(1);

	return;

}

//=============================================================================
// Name: DIOPortLoopbackTest()
// Desc: loop back test on various ports
// Parameters: void
// Return: void
//=============================================================================
void DIOPortLoopbackTest()
{

	int intBuff = 0; 
	int output_byte = 0; 
	int input_byte = 0; 
	int port = 0; 
    int input_port = 0;
    int output_port = 0;
    int i = 0;
    int errorcount=0;
    BYTE return_value = DE_NONE;

	do
	{
	 
	printf ( "Enter port number for input (0-%d): or  Enter -1 to go to Main menu\n ", DIOPORT_MAX-1 );
        fgets(input_buffer,20,stdin);
        sscanf(input_buffer,"%d",&input_port);
	
	if(input_port==-1) return;
	printf ( "Enter  port number for Output (0-%d): or Enter -1 to go to Main menu\n ", DIOPORT_MAX-1 );
	fgets(input_buffer,20,stdin);
        sscanf(input_buffer,"%d",&output_port);
	
	if(input_port==-1) return;
	
	if (( input_port > DIOPORT_MAX || input_port < 0 ) || (output_port > DIOPORT_MAX || output_port < 0 ) )
		{
			printf ( "ERROR: Valid Port Numbers are: (0-%d)\n ", DIOPORT_MAX );
			return;
		}
	
    	
	if(input_port ==DIOPORT_C  || input_port ==DIOPORT_F  )
	{
		if( portConfig[input_port]==0xF)
		{
		  printf("The port %d is not configured as input and the Port is in output mode \n",input_port);

		  return;
		}
		
	}
	else
	{
		if( portConfig[input_port]==1)
		{

		  printf("The port %d is not configured as input and the Port is in output mode \n",input_port);
		  return;
		}

		
	}

		
	if( portConfig[output_port]==0)
	{
	  printf("The port %d is not configured as output and the Port is in input mode \n",output_port);
	  return;
	}
	
	
	errorcount=0;
	for(intBuff=0; intBuff<=255;intBuff++)
	{  
           
	    output_byte=(BYTE) intBuff;
		
		if( (MPEGPIODIOOutputByte( bi, (unsigned char)output_port, output_byte ) != DE_NONE) )
		{
			dscGetLastError ( &errorParams );
			printf (  "MPEGPIODIOOutputByte() error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
			return;
		}
		
		if( (MPEGPIODIOInputByte ( bi, (unsigned char)input_port, &input_byte ) != DE_NONE) )
		{
				dscGetLastError ( &errorParams );
				printf (   "MPEGPIODIOInputByte() error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
				return;
		}

		 
		if(output_byte !=input_byte)
		{
			  printf("Error: Output: 0x%x  Input: 0x%x \n",output_byte,input_byte);
		          errorcount+=1;
		}
			
	}
	 printf("\nValues written : %d and Errors: %d \n",intBuff,errorcount);
	} while(1);
 return;
 }   

//=============================================================================
// Name: RegisterReadBack()
// Desc: Reads FPGA Register values 
// Parameters: Baseaddress
// Return: void
//=============================================================================

void registerReadBack(int baseaddress)
{

  int intbuff=0;
  char cValue = 0;
  int i;
  int pageno = 0;


	printf("Enter page number (0-15) or Enter -1 to go to Main menu \r\n");
	fgets(input_buffer,20,stdin);
        sscanf(input_buffer,"%d",&pageno);
        
    if(pageno==-1) return;

	printf("-------------------------------------------------------------------------------------------------------\r\n");
	printf("PageNo:   0   1   2   3   4   5   6   7   8   9   A   B   C   D   E   F\r\n");
	printf("-------------------------------------------------------------------------------------------------------\r\n");
	printf("    %x  ", pageno);
	
	for(i = 0; i <= 15; i++)
	{
		dscInp (baseaddress + (16 * pageno) + i, &cValue);
		printf("   %x", cValue);
   	}

	printf("\n\n");
}



