//=============================================================================
// (c) Copyright 2014 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: MPEGPIOLEDFunction.c   v1.00
//
// Dependency :	Universal Driver version 7.00
//				DSCUD.H - Include file.
//				
//				LIBDSCUD-7.00.a	- Library file for Linux 2.6.29-3.x
//						
//				DSCUD.LIB	- Static library file for Windows 
//						( USE VC++ 6.0 or Visual Studio 2005 )
//
// 
// Description: The application toggles on board LED whenever space bar key is pressed
//	
//=============================================================================
#include "DSCUD_demo_def.h"
#include "mpegpio.h"

//Global variable declarations
ERRPARAMS errorParams; // structure for returning error code and error string
DSCCBP dsccbp; // structure containing PCI board settings
BoardInfo *bi=NULL; //Structure containing board base address
char input_buffer[20];
MPEGPIOINIT Init; //structure containing board FPGA revision ID, major ID etc.
int LEDOn=1; // temp variables of size int

//=============================================================================
// Name: main()
// Desc: It calls universal driver LED functions to demonstrates use of LED function
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. LED toggling
//	    IV. Clean-up
//
//=============================================================================

int main ( void )
{
	int key=0;
	
   	memset ( &dsccbp, 0, sizeof(DSCCBP) );	
  	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//    STEPS TO FOLLOW:
	//
	//	  1. initialize the driver, using the driver version for validation
	//=========================================================================

	if ( (dscInit ( DSC_VERSION ) != DE_NONE) )
	{
		dscGetLastError ( &errorParams );
		printf ( "dscInit error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
		return 0;
	}
		
	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the DS-MPE-GPIO Board. This function passes the various
	//	   hardware parameters to the driver and initializes the hardware.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. set the board type to DSC_MPEGPIO for the DS-MPE-GPIO board
	//	   2. Set the pci_slot variable to zero 
	//     3. Call MPEGPIOInitBoard universal driver function to initialize the board
	//=========================================================================

     printf ( "\nDSC MPE-GPIO LED Demo\n" );
	 
	dsccbp.boardtype = DSC_MPEGPIO;
	dsccbp.pci_slot  = 0;

	 
	if(MPEGPIOInitBoard ( &dsccbp ,&Init ) !=DE_NONE)
	{
		dscGetLastError(&errorParams);
		printf("MPEGPIOInitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
		return 0;
	}
	bi = DSCGetBoardInfo ( dsccbp.boardnum ); 
	
    //=========================================================================
	//III. LED toggling
	// STEPS TO FOLLOW:
	//  1. Press space bar key to toggle the one board LED
	//
	//=========================================================================
	do
	{
		printf("Press space bar key to toggle LED or Press 'q' to quit  \n");
		while(1)
		{
			
			 key =getch();
			
			  if (key == 32)
			  {
				if(LEDOn)
				{
					MPEGPIOLED(bi,0); // LED Off
					LEDOn = 0;
				}
				else
				{
					MPEGPIOLED(bi,1); // LED On
					LEDOn = 1;
				}
			  }
			  else if(key == 'q')
			  {
				break;
			  }
						   
		}
		
	printf ("press Enter to repeat or 'q' to quit:");
   	fgets ( input_buffer, 20, stdin );

    } while ( input_buffer[0] != 'q' );
		
	//=========================================================================
	// V. CLEANUP
	//
	//	   Clean up any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the driver resources
	//=========================================================================

	dscFree ( );
	
	return 0;

} // end main()
