//=============================================================================
// (c) Copyright 2014 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: MPEGPIOPWMFunction.c  v1.00
//
// Dependency :	Universal Driver version 7.00
//				DSCUD.H - Include file.
//				
//				LIBDSCUD-7.00.a	- Library file for Linux 2.6.29-3.x
//						
//				DSCUD.LIB	- Static library file for Windows 
//						( USE VC++ 6.0 or Visual Studio 2005 )
//				
// Description: This sample code demonstrates the usage of UD function  
//				MPEGPIOPWMConfig().The user selects one of the
//              Pulse Width Modulators (PWM), the output polarity to use,
//              base frequency, and duty cycle of the modulation. 
//============================================================================
#include "DSCUD_demo_def.h"
#include "mpegpio.h"

//Global variable declarations
ERRPARAMS errorParams; // structure for returning error code and error string
DSCCBP dsccbp; // structure containing PCI board settings
BoardInfo *bi=NULL; //Structure containing board base address
MPEGPIOINIT Init; //structure containing board FPGA revision ID, major ID etc.
MPEGPIOPWM pwm;//structure containing PWM settings 
//=============================================================================
// Name: main()
// Desc: It calls universal driver PWM related functions to demonstrate PWM functionality 
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. PWM configuration 
//	    IV. Clean up
//
//=============================================================================

int main ( void )
{
	
    char input_buffer[20];
   	memset ( &dsccbp, 0, sizeof(DSCCBP) );
	memset ( &pwm, 0 ,sizeof(MPEGPIOPWM));

	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//    STEPS TO FOLLOW:
	//
	//	  1. initialize the driver, using the driver version for validation
	//=========================================================================

	if ( (dscInit ( DSC_VERSION ) != DE_NONE) )
	{
		dscGetLastError ( &errorParams );
		printf ( "dscInit error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
		return 0;
	}
		
	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the DS-MPE-GPIO Board. This function passes the various
	//	   hardware parameters to the driver and initializes the hardware.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. set the board type to DSC_MPEGPIO for the DS-MPE-GPIO board
	//	   2. Set the pci_slot variable to zero 
	//     3. Call MPEGPIOInitBoard universal driver function to initialize the board
	//=========================================================================

	printf ( "\nDSC MPE-GPIO PWM FUNCTIONS DEMO\n" );

	dsccbp.boardtype = DSC_MPEGPIO;
	dsccbp.pci_slot  = 0;

	 
	if(MPEGPIOInitBoard ( &dsccbp ,&Init ) !=DE_NONE)
	{
		dscGetLastError(&errorParams);
		printf("MPEGPIOInitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
		return 0;
	}
	bi = DSCGetBoardInfo ( dsccbp.boardnum ); 
	
   	
	//=========================================================================
	// III. PWM configuration
	//
	//	    Program the pulse width modulation. This function takes MPEGPIOPWM structure 
	//		as input with PWM no , frequency and duty cycle value.  This function configures the PWM circuit 
	//		and generates PWM signal
	//
	//=========================================================================
	do
	{
		printf("Select PWM no (0-3 ,Default = 0)");
		fgets(input_buffer,20,stdin);
		if(input_buffer[0]=='\n')
		{	
			pwm.Num = 0;
		}
		else
		{
			sscanf(input_buffer,"%d",&pwm.Num);
		}
		
		if(input_buffer[0] == 'q') return 0;
		
		printf("Select Output Frequency (1-50MHz ,Default = 100)");
		fgets(input_buffer,20,stdin);
		if(input_buffer[0]=='\n')
		{	
			pwm.Rate = 100;
		}
		else
		{
			sscanf(input_buffer,"%f",&pwm.Rate);
		}
		
		printf("Select Duty cycle value (1-100,Default = 50)");
		fgets(input_buffer,20,stdin);
		if(input_buffer[0]=='\n')
		{	
			pwm.Duty = 50;
		}
		else
		{
			sscanf(input_buffer,"%f",&pwm.Duty);
		}
		
		printf("Select Polarity (0 = pulse high, 1 = pulse low , Default = 0)");
		fgets(input_buffer,20,stdin);
		if(input_buffer[0]=='\n')
		{	
			pwm.Polarity = 0;
		}
		else
		{
			sscanf(input_buffer,"%d",&pwm.Polarity);
		}
		
		
		pwm.OutputEnable = 1;
		pwm.Run = 0;
		
		if(MPEGPIOPWMConfig(bi,&pwm) !=DE_NONE)
		{
			dscGetLastError(&errorParams);
			printf("MPEGPIOPWMConfig error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
			return 0;
		}
		
		printf("Press ENTER key to start PWM \n");
		fgets(input_buffer,20,stdin);
		if( MPEGPIOPWMStart(bi,pwm.Num) !=DE_NONE)
		{
			dscGetLastError(&errorParams);
			printf("MPEGPIOPWMStart error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
			return 0;
		}
		
		printf("Press ENTER key to stop PWM \n");
		fgets (input_buffer, 20, stdin );
		
		if(MPEGPIOPWMStop(bi,pwm.Num) !=DE_NONE)
		{
			dscGetLastError(&errorParams);
			printf("MPEGPIOPWMStop error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
			return 0;
		}

		if( MPEGPIOPWMReset(bi,pwm.Num) !=DE_NONE)
		{
			dscGetLastError(&errorParams);
			printf(" MPEGPIOPWMReset error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
			return 0;
		}
		
		printf ("press Enter to repeat or 'q' to quit:");
   		fgets ( input_buffer, 20, stdin );

    } while ( input_buffer[0] != 'q' );
	
	//=========================================================================
	// IV. CLEANUP
	//
	//	   Clean up any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the driver resources
	//=========================================================================

	dscFree ( );
	
	return 0;

} // end main()


