	//=============================================================================
// (c) Copyright 2014 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: MPEGPIOPullUpDownConfig.c  v1.0
//  Dependency :	Universal Driver version 7.00
//				DSCUD.H - Include file.
//				
//				LIBDSCUD-7.00.a	- Library file for Linux 2.6.29-3.x
//						
//				DSCUD.LIB	- Static library file for Windows 
//						( USE VC++ 6.0 or Visual Studio 2005 )
// Description: This function demonstrates use of MPEGPIOConfig() function . It configures the DIO pins
// 			    in Pull up or down mode .  
//============================================================================
#include "DSCUD_demo_def.h"
#include "mpegpio.h"


//Global variable declarations
ERRPARAMS errorParams; // structure for returning error code and error string
DSCCBP dsccbp; // structure containing PCI board settings
BoardInfo *bi=NULL;//Structure containing board base address
MPEGPIOINIT Init; //structure containing board FPGA revision ID, major ID etc.
//=============================================================================
// Name: main()
// Desc: It calls universal driver MPEGPIOConfig functions to demonstrate DIO pull up/down configuration
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. DIO Pull up/down configuration
//	    IV. Clean up
//
//=============================================================================

int main ( void )
{

	int intBuff = 0; // temp variables of size int
	char input_buffer[20];
  	
   	memset ( &dsccbp, 0, sizeof(DSCCBP) );
		 
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//    STEPS TO FOLLOW:
	//
	//	  1. initialize the driver, using the driver version for validation
	//=========================================================================

	if ( (dscInit ( DSC_VERSION ) != DE_NONE) )
	{
		dscGetLastError ( &errorParams );
		printf ( "dscInit error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
		return 0;
	}
	
	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the DS-MPE-GPIO Board. This function passes the various
	//	   hardware parameters to the driver and initializes the hardware.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. set the board type to DSC_MPEGPIO for the DS-MPE-GPIO board
	//	   2. Set the pci_slot variable to zero 
	//     3. Call MPEGPIOInitBoard universal driver function to initialize the board
	//=========================================================================

	printf ( "\nDSC MPE-GPIO Pull up/down Function Demo\n" );

	dsccbp.boardtype = DSC_MPEGPIO;
	dsccbp.pci_slot  = 0;
	 
	if(MPEGPIOInitBoard ( &dsccbp ,&Init ) !=DE_NONE)
	{
		dscGetLastError(&errorParams);
		printf("MPEGPIOInitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
		return 0;
	}
	bi = DSCGetBoardInfo ( dsccbp.boardnum ); 

	 //////////////////////////////////////////////////////////////////////////
	 //III. DIO Pull up/down configuration
	 //   1. Enter desired DIO configuration from followings
	 //   CFG1 = PORT A-C pull, 1 = up, 0 = down 
	 //   CFG2 = PORT D-F pull, 1 = up, 0 = down 
	 //	  2.Repeat until user types 'q' to exit
	 /////////////////////////////////////////////////////////////////////////
	
	 printf("DIO configuration details \n");
	 printf("CFG1 = PORT A-C pull, 1 = up, 0 = down \n");
	 printf("CFG2 = PORT D-F pull, 1 = up, 0 = down \n\n");

	 do
	 {			
		printf("Enter DIO configuration value CFG2-1 (default 0) :");
		fgets(input_buffer,20,stdin);
		if(input_buffer[0] == '\n')
		{
			intBuff = 0;
		}	
		else
		{  
			sscanf(input_buffer,"%d",&intBuff);
		}
		if( MPEGPIOConfig(bi,intBuff) !=DE_NONE)
		{
			dscGetLastError(&errorParams);
			printf ( "MPEGPIOConfig error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );		
			return 0;
		}
		printf("Press Enter to repeat or 'q' to quit \n");
		fgets(input_buffer,20,stdin);
	 }while ( input_buffer[0] != 'q' );

	//=========================================================================
    // IV. CLEANUP
    //
    //     Clean up any remnants left by the program and free the resources used
    //     by the driver.
    //
    //     STEPS TO FOLLOW:
    //
    //     1. free the driver resources
    //=========================================================================
	
	dscFree ( );
	
	return 0;

} // end main()

