///=============================================================================
// (c) Copyright 2014 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: MPEGPIOUserIntFunction.c  v1.00
//
// Dependency :	Universal Driver version 7.00
//				DSCUD.H - Include file.
//				
//				LIBDSCUD-7.00.a	- Library file for Linux 2.6.29-3.x
//						
//				DSCUD.LIB	- Static library file for Windows 
//						( USE VC++ 6.0 or Visual Studio 2005 )
//				
// Description: This sample code demonstrates the usage of UD function
//				MPEGPIOUserInterruptConfig(),The  'userfun()' is called each time when
//              a timer or DIO generates an interrupt. The userfun() increments a count value to show number of times interrupt generated. 
//              The count value is printed every second.
//============================================================================
#include "DSCUD_demo_def.h"
#include "mpegpio.h"
int count =0;
void userfun (void *param)
{
   count ++;
}
//Global variable declarations
ERRPARAMS errorParams; // structure for returning error code and error string
DSCCBP dsccbp; // structure containing board settings
BoardInfo *bi; //Structure containing board base address
MPEGPIOINIT Init;//structure containing board FPGA revision ID, major ID etc.
MPEGPIOUSERINT inter; // Structure containing interrupt settings
MPEGPIOCTR counter; // structure containing counter settings

//=============================================================================
// Name: main()
// Desc: This calls User Interrupt function and demonstrates Counter and DIO interrupt
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. UserInt Settings
//	    IV. Cleanup
//
//=============================================================================
int main ( void )
{

	int intBuff = 0; // temp variables of size int
	char input_buffer[20];
	int port=0,dir=0;
	DWORD baseadd=0;
	memset ( &dsccbp, 0, sizeof(DSCCBP) );
  
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//    STEPS TO FOLLOW:
	//
	//	  1. initialize the driver, using the driver version for validation
	//=========================================================================

	if ( (dscInit ( DSC_VERSION ) != DE_NONE) )
	{
		dscGetLastError ( &errorParams );
		printf ( "dscInit error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the DS-MPE-GPIO Board. This function passes the various
	//	   hardware parameters to the driver and initializes the hardware.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. set the board type to DSC_MPEGPIO for the DS-MPE-GPIO board
	//	   2. Set the pci_slot variable to zero 
	//     3. Call MPEGPIOInitBoard universal driver function to initialize the board
	//=========================================================================

		printf ( "\nDSC MPE-GPIO UserInt FUNCTIONS DEMO\n" );

		dsccbp.boardtype = DSC_MPEGPIO;
		dsccbp.pci_slot  = 0;
		 
		if(MPEGPIOInitBoard ( &dsccbp ,&Init ) !=DE_NONE)
		{
			dscGetLastError(&errorParams);
			printf("MPEGPIOInitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
			return 0;
		}
		bi = DSCGetBoardInfo ( dsccbp.boardnum ); 
  		//=========================================================================
		// III. USER INT INITIALIZATION and Run
		//
		//		Sets up the rate,interrupt source for RMM1616UserInterruptSet()
		//=========================================================================
		do
		{
			count = 0;
			inter.IntFunc=userfun;
			inter.Mode=0;
			inter.Enable=1;
			printf("Enter Interrupt source (1-5; 1 = digital I/O; 2-5 = counter/timer 0-3 output ) default =4 ");
			fgets(input_buffer,20,stdin);
			if(input_buffer[0] == '\n')
			{
				inter.Source=4;
			}
			else
			{
				sscanf(input_buffer,"%d",&inter.Source);
			}
			
			if(MPEGPIOUserInterruptConfig(bi,&inter) !=DE_NONE)
			{          
				dscGetLastError ( &errorParams );
				printf ( "MPEGPOUserInterruptConfig error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
				return 0;
			}
			
			if(MPEGPIOUserInterruptRun(bi,inter.Source) !=DE_NONE)
			{
				dscGetLastError ( &errorParams );
				printf ( "MPEGPIOUserInterruptRun error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
				return 0;
			}
		
			if(inter.Source !=1)
			{
				printf("Enter Interrupt source frequency rate (1-50MHZ) (default = 1000)");
				fgets(input_buffer,20,stdin);
				if(input_buffer[0] == '\n')
				{
					counter.Rate = 1000;
				}
				else
				{
					sscanf(input_buffer,"%f",&counter.Rate);
				}
				counter.Num = inter.Source-2 ;
				counter.OutEn = 0;
				counter.OutPol= 0;
				counter.ctrOutWidth = 0 ;
				if(MPEGPIOCounterSetRate(bi,&counter) !=DE_NONE)
				{
					dscGetLastError ( &errorParams );
					printf ( "RMM1616CounterSetRate error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
					return 0;
				}
			}
			else
			{
			  
			  port = 0; //PORTA 		   

			   printf("Enter PORTA direction (0-input,1-output default:1)");
			   fgets(input_buffer,20,stdin);
			   if(input_buffer[0] == '\n')
			   {
				dir = 1;
			   }
			   else
			   {
				sscanf(input_buffer,"%d",&dir);
			   }

			   if(MPEGPIODIOConfig(bi,port,dir) !=DE_NONE)
			   {
				dscGetLastError ( &errorParams );
					printf ( "MPEGPIODIOConfig error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
					return 0;
				   }

			
			}
			
			printf("Press ENTER key to terminate the application \n");
			while( !kbhit())
			{
				dscSleep(1000);
				printf("Count value %d \r",count);
				fflush(stdout);
			}
			fgets(input_buffer,20,stdin);
			printf("Press ENTER key to terminate the application \n");
			inter.Enable= 0;
			if(MPEGPIOUserInterruptCancel(bi,inter.Source) !=DE_NONE)
			{
			  printf("Hello\n");
				dscGetLastError ( &errorParams );
				printf ( "MPEGPIOUserInterruptCancel error: %s %s\n", dscGetErrorString ( errorParams.ErrCode ), errorParams.errstring );
				return 0;
			}
			printf ("press Enter to repeat or 'q' to quit:");
   		    fgets ( input_buffer, 20, stdin );

    } while ( input_buffer[0] != 'q' );

	//=========================================================================
	// V. CLEANUP
	//
	//	   Clean up any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the memory allocated for sample values
	//	   2. free the driver resources
	//=========================================================================

	dscFree ( );
	return 0;

} // end main()


#ifdef _WIN32_WCE
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	main ( );

	return 0;
}

#endif

